// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	ExprColumnGenerator.h:	--- create new table columns
//
//	DESCRIPTION:
//		A class ExprColumnGenerator is defined which populates new columns
//		from arithmetic expressions involving other columns in a table.
//
//	CHANGE HISTORY:
//		UL, 1999-04-27: 1.0.0: - first version from old ColumnGenerator class
//
//	AUTHOR:
//		U. Lammers (UL), Wed Aug 27 11:48:12 MET DST 1997
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/service/ExprColumnGenerator.h,v 1.2 2001/08/10 16:14:59 ulammers Exp $
//

#ifndef EXPR_COLUMN_GENERATOR_H
#define EXPR_COLUMN_GENERATOR_H

#include "ColumnGenerator.h"
#include "DalTableDescriptor.h"

class ExprColumnGenerator : public virtual ColumnGenerator {
public:
	ExprColumnGenerator() : ColumnGenerator(),
		_dalTableDescriptor(0), _evaluator(0) {}
		//
		//	PURPOSE:
		//		Create a dummy ExprColumnGenerator object


	ExprColumnGenerator(const std::string &expression,
						const std::string &columnName = "TEMP",
						Dal::DataType dataType = Dal::Real64);
		//
		//	PURPOSE:
		//		Create a ExprColumnGenerator object with an arithmetic
		//		expression
		//
		//	ARGUMENTS:
		//		In:
		//			expression: the arithmetic expression subsequent
		//						doIt() invocations will
		//						use to create new table columns;
		//						<expression> must adhere to the syntax rules
		//						detailed in the User documentation, otherwise a
		//						fatal error condition is raised
		//			columnName: name of new column
		//			dataType  : type of new column
		//


	void doIt(const Table &inTable, Table &outTable) const;
		//
		//	PURPOSE:
		//		Create a new column according to the arithmetic expression
		//		this object was instantiated with;
		//
		//	ARGUMENTS:
		//		In:
		//			inTable:	the table containing the columns and attributes
		//						referred to	in the arithmetic expression
		//
		//		InOut:
		//			outTable:	the table to which the newly created column
		//						<columnName> will be added; <outTable> and
		//						<inTable> can be identical.

	virtual ~ExprColumnGenerator();
 		//
		//	PURPOSE:
		//		destroy the object constructed with either of the above
		//		constructors

private:
	mutable std::string			_expression;
	mutable DalTableDescriptor	*_dalTableDescriptor;
	mutable Evaluator			*_evaluator;
};

#endif
