// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	ServiceF90cxxGlue.h	--- the Fortran90/C++ interface of selector-services
//
//	DESCRIPTION:
//		This file acts as the interface between the filter/arrayeval services
//		declared in corresponding f90-modules and their implementation in
//		C++.
//
//	CHANGE HISTORY:
//		UL, 1998-03-25: 1.0.0: - skeleton
//		UL, 1998-05-28: 1.1.0: - added ArrayEvaluator
//		UL, 2002-07-09: 1.2.0: - added ExprColumnGenerator
//		UL, 2002-07-31: 1.3.0: - added selectReleaseMemory
//
//	AUTHOR:
//		U. Lammers, Wed Mar 25 11:15:00 MET 1998
//
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/service/ServiceF90cxxGlue.h,v 1.3 2002/07/31 12:27:36 ulammers Exp $
//

#ifndef SERVICE_F90CPP_H
#define SERVICE_F90CPP_H

#include "common.h"
#include "f90cxx.h"
#include "Dal.h"

//
//	-------------------------( Filter )-----------------------------------
//
extern "C" void CXXFUNC(initfilter, initFilter)
							(const char *expression,
							 Integer3 expressionLength);

extern "C" Integer3 CXXFUNC(dofilter, doFilter)
							(Table *const *table);

extern "C" Integer3 CXXFUNC(doselectmodify, doSelectModify)
							(Table *const *table,
							 const char *flagColumnName,
							 const Integer2 *flagBitPos,
							 Integer3 flagColumnLength);

extern "C" Integer3 CXXFUNC(doselectconst, doSelectConst)
							(const Table *const *table,
							 Dope<Integer3, 1> *fromRow,
							 Dope<Integer3, 1> *toRow);

extern "C" void CXXFUNC(selectreleasememory, selectReleaseMemory)
							(Dope<Integer3, 1> *fromRow,
							 Dope<Integer3, 1> *toRow);

//
//	-----------------------( ArrayEvaluator )-----------------------------
//
extern "C" void CXXFUNC(initarrayeval, initArrayEval)
							(const char *expression,
							 Integer3 expressionLength);

extern "C" void CXXFUNC(doarrayeval, doArrayEval)
							(Dope<Array *, 1> *inArray,
							 Integer3 *lenInArray,
							 Array **outArray);

//
//	-----------------------( ExprColumnGenerator )-----------------------------
//
extern "C" void CXXFUNC(initexprcolgen, initExprColGen)
							(const char *expression,
							 const char *colName,
							 const Integer3 *colType,
							 Integer3 expressionLength,
							 Integer3 colNameLength);

extern "C" void CXXFUNC(doexprcolgen, doExprColGen)
							(const Table **inTable,
							 Table **outTable);

#endif

