// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	nullfiltertest: test the filtering on columns with NULL values
//
//	UL, Mon Jul  3 14:03:22 MET DST 2000
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/test/nullfiltertest.cc,v 1.4 2002/12/17 17:48:12 jbakker Exp $
//

#include <cassert>
#include <iostream>
#include "DataFilter.h"
#include "Dal.h"
#include "NAN.h"

int main()
{
	const unsigned NROWS = 10;
	const std::string qualColName = "FLAGS";
	const long UI16	= 99;
	const long UI32	= 999;

	//
	//	create a test set
	//
	DataSet *set = dataSetServer->open("nulldattest.ds", DataSet::Create);
	Table *table = set->addTable("NULL", NROWS);
	Column *ci16 = table->addColumn("I16", Dal::Int16),
		   *ci32 = table->addColumn("I32", Dal::Int32),
		   *cr32 = table->addColumn("R32", Dal::Real32),
		   *cr64 = table->addColumn("R64", Dal::Real64);

	ci16->nullValue(UI16);
	ci32->nullValue(UI32);
	ColumnData *cdi16 = ci16->data(),
			   *cdi32 = ci32->data(),
			   *cdr32 = cr32->data(),
			   *cdr64 = cr64->data();
	int16 *i16 = cdi16->int16Data();
	int32 *i32 = cdi32->int32Data();
	real32 *r32 = cdr32->real32Data();
	real64 *r64 = cdr64->real64Data();

	for (unsigned n=0; n<NROWS; ++n) {
		i16[n] = n<1 ? UI16 : n;
		i32[n] = n<2 ? UI32 : n;
		r32[n] = n;
		r64[n] = n;
		if (n<3)
			NaN.set(r32[n]);
		if (n<4)
			NaN.set(r64[n]);
	}

	delete cdi16;
	delete cdi32;
	delete cdr32;
	delete cdr64;

	DataFilter filter01("ISNULL(I16)");
	assert(filter01.doSelect(*table, qualColName, 0) == 1);

	DataFilter filter02("ISNULL(I32)");
	assert(filter02.doSelect(*table, qualColName, 0) == 2);

	DataFilter filter03("ISNULL(R32)");
	assert(filter03.doSelect(*table, qualColName, 0) == 3);

	DataFilter filter04("ISNULL(R64)");
	assert(filter04.doSelect(*table, qualColName, 0) == 4);

	//	complex
	DataFilter filter05("!ISNULL(I16) && !ISNULL(I32) && "
						"!ISNULL(R32) && !ISNULL(R64)");
	assert(filter05.doSelect(*table, qualColName, 0) == 6);

	dataSetServer->close(set);

	std::cout<<"All tests successfully passed.\n";
	return 0;
}
