// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	polygonbech: test performance of polygon filter
//
//	UL, Sat Aug  4 16:17:56 MET DST 2001
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/test/polybench.cc,v 1.3 2009/06/26 15:51:59 aibarra Exp $
//
#include <string>
#include <vector>
#include <iostream>
#include "Evaluator.h"
#include "TriangulatablePolygon.h"
#include "Random.h"
#include <stdlib.h>

int main(int argc, char *argv[])
{
	if (argc != 3) {
		std::cerr<<"Usage: "<<argv[0]<<" <npolypoints> <nruns>\n";
		return 1;
	}

	unsigned NV = atoi(argv[1]),
			 N = atoi(argv[2]);

	std::vector<double> xCoords,
						yCoords;

	randGen.max(100.);
	for (unsigned n=0; n<NV; ++n) {
		xCoords.push_back(randGen.rand());
		yCoords.push_back(randGen.rand());
	}

	TriangulatablePolygon poly(xCoords, yCoords);
	std::string expr = poly.toSelectExpression("X", "Y");
	std::cout<<expr<<std::endl;
	Evaluator evaluator(expr);	//	construct evaluator

	//	check identifiers
	NumericSymbols idents = evaluator.numericIdentifiers();
	assert(idents.name.size() == 2);

	randGen.max(100.);
	unsigned nin = 0, nout = 0;
	for (unsigned n=0; n<N; ++n) {
		randGen.max(100.);
		*idents.valuePtr[0] = randGen.rand();
		*idents.valuePtr[1] = randGen.rand();

		if (evaluator.evalLogic())
			++nin;
		else
			++nout;
	}
	std::cout<<"inside : "<<nin<<std::endl;
	std::cout<<"outside: "<<nout<<std::endl;

	return 0;
} // main()
